﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace DynamicColumns.Attached
{
    using System.Windows;
    using System.Windows.Data;

    using DynamicColumns.Data;

    using Telerik.Windows.Controls;

    public class RadGridViewDataHelper : DependencyObject
    {
        #region Fields

        private RadGridView radGridView;

        #endregion Fields

        #region Dependency Properties


        public static readonly DependencyProperty TemplateSelectorProperty = DependencyProperty.Register("TemplateSelector", typeof(TemplateSelector), typeof(RadGridViewDataHelper), new PropertyMetadata(TemplateSelectorChanged));

        private static void TemplateSelectorChanged(DependencyObject d, DependencyPropertyChangedEventArgs e)
        {
        }

        public static readonly DependencyProperty HelperProperty = DependencyProperty.RegisterAttached("Helper", typeof(RadGridViewDataHelper), typeof(RadGridView), new PropertyMetadata(HelperChanged));

        #endregion Dependency Properties

        #region Dependency Property Methods

        public static void SetHelper(RadGridView radGridView, RadGridViewDataHelper value)
        {
            radGridView.SetValue(HelperProperty, value);
        }

        public static RadGridViewDataHelper GetHelper(RadGridView itemsControl)
        {
            return (RadGridViewDataHelper)itemsControl.GetValue(HelperProperty);
        }

        public TemplateSelector TemplateSelector
        {
            get
            {
                return this.GetValue(TemplateSelectorProperty) as TemplateSelector;
            }

            set
            {
                this.SetValue(TemplateSelectorProperty, value);
            }
        }

        #endregion Dependency Property Methods

        #region Dependency Property Handler

        private static void HelperChanged(DependencyObject d, DependencyPropertyChangedEventArgs e)
        {
            if (d is RadGridView && e.NewValue is RadGridViewDataHelper)
            {
                RadGridViewDataHelper radGridViewHelper = ((RadGridViewDataHelper)e.NewValue);

                radGridViewHelper.Initialize((RadGridView)d);
            }
        }

        #endregion Dependency Property Handler

        private void RadGridView_AutoGeneratingColumn(object sender, GridViewAutoGeneratingColumnEventArgs e)
        {
            RadGridView gridView = sender as RadGridView;

            if (gridView != null)
            {
                    this.InitializeColumnHeader(e.Column);

                    this.InitializeCellTemplate(e.Column);
            }
        }

        private void InitializeColumnHeader(GridViewColumn gridViewColumn)
        {
            ColumnMetadata columnMetadata = DataCreator.ColumnMetadatas.FirstOrDefault(c => c.Name == gridViewColumn.UniqueName);

            if (columnMetadata != null)
            {
                gridViewColumn.Header = columnMetadata.Alias;
            }
        }

        private void InitializeCellTemplate(GridViewColumn gridViewColumn)
        {
            gridViewColumn.CellTemplateSelector = this.TemplateSelector;
        }

        private void Initialize(RadGridView radGridView)
        {
            this.radGridView = radGridView;

            this.radGridView.AutoGeneratingColumn += RadGridView_AutoGeneratingColumn;
        }
    }
}
