﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace DynamicColumns.Data
{
    using System.Data;

    public static class DataCreator
    {
        static Random columnNumberRandom = new Random();

        static Random columnPrecisionRandom = new Random();

        static Random rowCountRandom = new Random();

        static Random doubleRandom = new Random();

        public static List<ColumnMetadata> ColumnMetadatas
        {
            get; private set;
        }
 
        public static DataTable Create()
        {
            DataTable dataTable = new DataTable();

            AddColumns(dataTable);

            AddData(dataTable);

            return dataTable;
        }

        private static void AddColumns(DataTable dataTable)
        {
            int columnCount = columnNumberRandom.Next(10, 30);

            ColumnMetadatas = new List<ColumnMetadata>();

            for (int i = 0; i < columnCount; i++)
            {
                string columnName = string.Format("Column{0}", i);

                int columnPrecision = columnPrecisionRandom.Next(5);

                string columnAlias = string.Format("Column #{0}, Precision {1}", i, columnPrecision);

                ColumnMetadatas.Add(new ColumnMetadata() { Alias = columnAlias, Name = columnName, Precision = columnPrecision});

                DataColumn dataColumn = new DataColumn(columnName, typeof(double));

                dataTable.Columns.Add(dataColumn);
            }
        }

        private static void AddData(DataTable dataTable)
        {
            int rowCount = rowCountRandom.Next(200);

            for (int i = 0; i < rowCount; i++)
            {
                List<object> list = new List<object>();

                for (int j = 0; j < dataTable.Columns.Count; j++)
                {
                    double value = doubleRandom.NextDouble() * 100;
                    list.Add(value);
                }

                dataTable.Rows.Add(list.ToArray());
            }
        }
    }
}
