﻿
namespace DynamicColumns.ViewModels
{
    using System;
    using System.Collections.Specialized;
    using System.ComponentModel;
    using System.Linq.Expressions;

    public class ViewModelBase : INotifyPropertyChanged, INotifyPropertyChanging
    {
        #region Events

        #region INotifyPropertyChanged Members

        public event PropertyChangedEventHandler PropertyChanged;

        public event PropertyChangingEventHandler PropertyChanging;

        #endregion INotifyPropertyChanged Members

        #endregion Events

        #region Methods

        #endregion Methods

        #region Helper Methods

        protected void RaisePropertyChanged(string propertyName)
        {
            this.OnPropertyChanged(propertyName);
        }

        private void OnPropertyChanged<T>(Expression<Func<T>> expression)
        {
            MemberExpression memberExpression = (MemberExpression)expression.Body;
            this.OnPropertyChanged(memberExpression.Member.Name);
        }

        protected void RaisePropertyChanged<T>(Expression<Func<T>> expression)
        {
            this.OnPropertyChanged(expression);
        }


        protected virtual void OnPropertyChanged(string propertyName)
        {
            if (this.PropertyChanged != null)
            {
                this.PropertyChanged(this, new PropertyChangedEventArgs(propertyName));
            }
        }

        protected void RaisePropertyChanging(string propertyName)
        {
            this.OnPropertyChanging(propertyName);
            if (this.PropertyChanging != null)
            {
                this.PropertyChanging(this, new PropertyChangingEventArgs(propertyName));
            }
        }

        protected void RaisePropertyChanging<T>(Expression<Func<T>> expression)
        {
            MemberExpression memberExpression = (MemberExpression)expression.Body;
            this.OnPropertyChanging(memberExpression.Member.Name);
        }

        protected virtual void OnPropertyChanging(string propertyName)
        {
            if (this.PropertyChanging != null)
            {
                this.PropertyChanging(this, new PropertyChangingEventArgs(propertyName));
            }
        }


        protected void Register<V>(V viewModel, PropertyChangedEventHandler handler)
            where V : class, INotifyPropertyChanged
        {
            if (viewModel != null)
            {
                viewModel.PropertyChanged += handler;
            }
        }

        protected void Unregister<V>(V viewModel, PropertyChangedEventHandler handler)
            where V : class, INotifyPropertyChanged
        {
            if (viewModel != null)
            {
                viewModel.PropertyChanged -= handler;
            }
        }

        protected void Register<V>(NotifyCollectionChangedEventArgs e, PropertyChangedEventHandler handler)
            where V : class, INotifyPropertyChanged
        {
            if (e.NewItems != null && e.NewItems.Count > 0)
            {
                foreach (object item in e.NewItems)
                {
                    if (item is V)
                    {
                        V viewModel = (V)item;
                        this.Register(viewModel, handler);
                    }
                }
            }

            if (e.OldItems != null && e.OldItems.Count > 0)
            {
                foreach (object item in e.OldItems)
                {
                    if (item is V)
                    {
                        V viewModel = (V)item;
                        this.Unregister(viewModel, handler);
                    }
                }
            }
        }

        #endregion Helper Methods
    }
}
